import re
from json.decoder import JSONDecodeError
import bs4
import requests
from requests.cookies import RequestsCookieJar


class AuthorizationException(Exception):
    pass


def parse_cookie(cookie_string: str) -> dict:
    return {
        (sp := x.split("="))[0]: sp[1] for x in cookie_string.split("; ")
    }


def _get_cookies(login: str, password: str) -> RequestsCookieJar:

    response = requests.get(
        "https://api.librus.pl/OAuth/Authorization?client_id=46&response_type=code&scope=mydata"
    )

    cookies = {}

    for cookie_string in response.request.headers["Cookie"].split("; "):

        data = cookie_string.split("=")

        cookies[data[0]] = data[1]

    response = requests.post(
        "https://api.librus.pl/OAuth/Authorization?client_id=46",
        data={
            "action": "login",
            "login": login,
            "pass": password
        },
        cookies=cookies
    )

    try:
        response_json = response.json()
    except JSONDecodeError:
        raise AuthorizationException("something went wrong")

    if response_json["status"] != "ok":
        raise AuthorizationException(response.json()["errors"])

    response = requests.get(
        "https://api.librus.pl/OAuth/Authorization/PerformLogin?client_id=46",
        cookies=cookies,
        headers={
            "Referer": "https://portal.librus.pl/rodzina"
        }
    )

    return response.cookies


def get_cookies(login: str, password: str) -> dict:
    return dict(_get_cookies(login, password))


def get_csrf_token(cookies: dict, parent: bool = False) -> str:
    response = requests.get(
        f"https://synergia.librus.pl/{'rodzic' if parent else 'uczen'}/index",
        cookies=cookies
    )

    response = re.search(
        'var csrfTokenValue = "(.*)";',
        bs4.BeautifulSoup(
            response.text, "html.parser"
        ).select_one(
            "script[type='text/javascript']:not(script[src])"
        ).text
    )

    if response is not None:
        return response.groups()[0]

    raise AuthorizationException("CSRF token not found")


def get_login_history(cookies: dict, parent: bool = False) -> dict:
    response = bs4.BeautifulSoup(
        requests.get(
            f"https://synergia.librus.pl/{'rodzic' if parent else 'uczen'}/index",
            cookies=cookies
        ).text,
        "html.parser"
    )

    title_strig = re.sub(
        "<.{1,4}>",
        " ",
        response.select_one("span.tooltip").attrs["title"]
    )

    regex_pattern =\
        r'(\d{4}-\d{1,2}-\d{1,2}) (\d{1,2}:\d{1,2}:\d{1,2}), IP: ([\d.]{7,15})'

    ip_info = list(map(
        lambda x: [
            {"date": entry[0], "time": entry[1], "ip": entry[2]} for entry in re.findall(regex_pattern, x)
        ],

        list(filter(
            lambda x: len(set(x)) > 1,
            re.split("ostatnie.{0,4}udane logowania:", title_strig)
        ))
    ))

    return {
        "successful": ip_info[0],
        "failed": ip_info[1]
    }



import bs4
import requests


def get_schedule(cookies: dict, csrf_token: str = None, week: str = None) -> dict:

    response = requests.post(
        url="https://synergia.librus.pl/przegladaj_plan_lekcji",
        cookies=cookies,
        data={
            "requestkey": csrf_token,
            "tydzien": week,
            "pokaz_zajecia_zsk": "on",
            "pokaz_zajecia_ni": "on"
        }
    )

    response = bs4.BeautifulSoup(
        response.text, "html.parser"
    )

    hour_to_lesson = {
        line.select_one("th").text[:5]: line.select_one("td").text
        for line in response.select("tr[class=line1]")
    }

    schedule = {}

    for lesson in response.select("td[data-date][class=line1]"):

        if (lesson_name := lesson.select_one("b")) is None:
            continue

        date = (attrs := lesson.attrs)["data-date"]
        lesson_time = (attrs["data-time_from"], attrs["data-time_to"])

        if date not in schedule:
            schedule[date] = {}

        tags = bs4.BeautifulSoup(str(lesson), "html.parser").select(
            "td[data-date][class=line1] > :not(:last-child)")

        schedule[date][hour_to_lesson[lesson_time[0]]] = {
            "subject": lesson_name.text.strip(),
            "time": lesson_time,
            "tags": [x.text for x in tags]
        }

    lesson_hours = {}

    for h in response.select("table.decorated.plan-lekcji tr.line1"):

        data = h.select("td:first-child, th")
        lesson_hours[data[0].text] = data[1].text.replace("\xa0", "")

    return {
        "days": schedule,
        "hours": lesson_hours
    }


def get_schedule_weeks(cookies: dict) -> list:
    return list(map(
        lambda x: x.attrs["value"],
        bs4.BeautifulSoup(
            requests.get(
                url="https://synergia.librus.pl/przegladaj_plan_lekcji",
                cookies=cookies
            ).text,
            "html.parser"
        ).select("select[id='tydzien'] > option")
    ))



import re
import bs4
import requests
from dataclasses import dataclass


def process_message_tr(row: bs4.element.Tag) -> dict:
    row_cels = row.select("td")

    data = {
        "nadawca": row_cels[2].text,
        "temat": row_cels[3].select_one("a[href]").text,
        "data": row_cels[4].text,
        "href": (href := row.select_one(
            "a[href^='/wiadomosci']").attrs["href"]),
        "id": re.findall("/wiadomosci/././([0-9]+)/.*", href)[0],
        "new": "style" in row_cels[2].attrs,
        "files": row_cels[1].select_one("img") is not None,
        "tags": None
    }

    tags = row_cels[3].select("span")

    if tags is None:
        return data

    data["tags"] = {tag.text for tag in tags}

    return data


def get_messages(
    cookies: dict, *,
    archive: bool = False,
    sent: bool = False,
    deleted: bool = False,
    csrf_token: str = None,
    person: str = "-",
    page: str = "0"
) -> dict:

    data = {
        "requestkey": csrf_token,
        "filtrUzytkownikow": "0",
        "idPojemnika": "105",
        "opcja_zaznaczone_g": "0",
        "filtr_uzytkownikow": person,
        "sortujTabele[tabeleKolumna]": "3",
        "sortujTabele[tabeleKierunek]": "1",
        "sortujTabele[tabelePojemnik]": "105",
        "sortowanie[105][0]": "",
        "sortowanie[105][1]": "",
        "sortowanie[105][2]": "",
        "opcja_zaznaczone_d": "0",
        "numer_strony105": page,
        "porcjowanie_pojemnik105": "105",
        "poprzednia": "5"
    }

    url = "https://synergia.librus.pl/wiadomosci_archiwum" if archive else "https://synergia.librus.pl/wiadomosci_aktualne"

    url = "https://synergia.librus.pl/wiadomosci/6" if sent else url

    url = "https://synergia.librus.pl/wiadomosci/7" if deleted else url

    response_object = requests.post(
        url,
        cookies=cookies,
        headers={
            "Referer": "https://synergia.librus.pl/wiadomosci_archiwum" if archive else "https://synergia.librus.pl/wiadomosci_aktualne"
        },
        data=data
    )

    response = bs4.BeautifulSoup(response_object.text, "html.parser")

    rows = response.select('table[class="decorated stretch"] > tbody > tr')

    messages = []

    if not (len(rows) == 1 and rows[0].select_one("td").text == "Brak wiadomości"):

        for row in rows:
            messages.append(process_message_tr(row))

    pagination = response.select_one("div.pagination > span")

    if pagination is not None:
        pagination = re.match(
            r"Strona (.*) z (.*)",
            pagination.text.replace('\xa0', " ")
        ).groups()

    else:
        pagination = (1, 1)

    return {
        "pagination": tuple(map(int, pagination)),
        "messages": messages,
    }


def read_message(cookies: dict, href: str) -> dict:

    response_object = requests.get(
        url="https://synergia.librus.pl" + href,
        cookies=cookies,
        headers={
            "Referer": "https://synergia.librus.pl/wiadomosci_archiwum"
        }
    )

    response = bs4.BeautifulSoup(
        response_object.text,
        "html.parser"
    )

    information = list(map(
        lambda x: x.text,
        response.select("td.left:not(:first-child)")
    ))

    files = "".join([
        x.attrs["onclick"].replace("\\", "")
        for x in response.select(
            "img[src='/assets/img/homework_files_icons/download.png']"
        )
    ])

    files = re.findall("/wiadomosci/pobierz_zalacznik/[0-9]+/[0-9]+", files)

    filenames = [
        x.find_parent("td").text.strip()
        for x in response.select('img[src^="/assets/img/filetype_icons"]')
    ]

    information = ["Użytkownik", *information]

    return {
        "nadawca": information[-4],
        "temat": information[-3],
        "data": information[-2],
        "data_odczytania": information[-1],
        "tresc": response.select_one("div.container-message-content").text,
        "files": list(zip(filenames, files))
    }


def get_senders_id(cookies: dict, *, archive: bool = False) -> list:

    response = requests.post(
        "https://synergia.librus.pl/wiadomosci_archiwum"
        if archive else "https://synergia.librus.pl/wiadomosci_aktualne",
        cookies=cookies,
        headers={
            "Referer": "https://synergia.librus.pl/wiadomosci"
            if archive else "https://synergia.librus.pl/wiadomosci_aktualne"
        }
    )

    response = bs4.BeautifulSoup(response.text, "html.parser")

    return list(map(
        lambda x: (x.attrs["value"], x.text),
        response.select("select[name='filtr_uzytkownikow'] > option")
    ))


def delete_messages(cookies: dict, messages: list, csrf_token: str, archive: bool = False) -> str:

    data = {
        "wiadomosciLista[]": messages,
        "folder": "5",
        "czyArchiwum": "0" if not archive else "1"
    }

    response = requests.post(
        "https://synergia.librus.pl/usun_wiadomosc",
        cookies=cookies,
        data=data,
        headers={
            "requestkey": csrf_token
        }
    )

    return response.text


def recover_messages(cookies: dict, messages: list, csrf_token: str, archive: bool = False) -> str:

    data = {
        "wiadomosciLista[]": messages,
        "folder": "7",
        "czyArchiwum": "0" if not archive else "1"
    }

    response = requests.post(
        "https://synergia.librus.pl/przywroc_wiadomosc",
        cookies=cookies,
        data=data,
        headers={
            "requestkey": csrf_token
        }
    )

    return response.text


@dataclass()
class Recipient():
    name: str
    recipient_id: str
    recipient_type: str


@dataclass()
class RecipientGroup:

    recipient_type: str
    is_virtual_classes: str = "false"
    group: str = "0"
    class_id: str = None

    def get_recipients(self, cookies: dict) -> list[Recipient]:

        response = requests.post(
            "https://synergia.librus.pl/getRecipients",
            cookies=cookies,
            data={
                "typAdresata": self.recipient_type,
                "czyWirtualneKlasy": self.is_virtual_classes == "true",
                "idGrupy": self.group,
                "klasa_rada_rodzicow": self.class_id,
                "klasa_opiekunowie": self.class_id,
                "klasa_rodzice": self.class_id,
            }
        )

        response = bs4.BeautifulSoup(
            response.text,
            "html.parser"
        )

        labels = response.select("label")

        return [Recipient(label.text.strip(), label.attrs['for'].split("_")[1], self.recipient_type) for label in labels]


def get_recipient_groups(cookies: dict) -> list[RecipientGroup]:

    response = bs4.BeautifulSoup(
        requests.get("https://synergia.librus.pl/wiadomosci/2/5", cookies=cookies).text,
        "html.parser"
    )

    inputs = response.select(
        "input[name='adresat']"
    )

    recipients = []

    for inp in inputs:

        onclick = inp.attrs["onclick"][:-2].split("(")[-1]

        onclick = [x.strip('"') for x in re.split(" ?, ?", onclick)]

        recipients.append(RecipientGroup(*onclick))

    return recipients


def send_message(
    cookies: dict,
    csrf_token: str,
    recipinet: Recipient,
    title: str,
    content: str
) -> None:

    data = {
        "requestkey": csrf_token,
        "adresat": recipinet.recipient_type,
        "DoKogo[]": recipinet.recipient_id,
        "DoKogo_hid[]": recipinet.recipient_id,
        "temat": title,
        "tresc": content,
        "wyslij": "Wyślij",
    }

    requests.post(
        "https://synergia.librus.pl/wiadomosci/5",
        cookies=cookies,
        headers={
            "Referer": "https://synergia.librus.pl/wiadomosci/2/5"
        },
        data=data
    )


def manage_tags_on_messages(
    cookies: dict,
    messages: list[str],
    tag_id: str,
    *,
    archive: bool = False,
    delete: bool = False
) -> int:

    if not messages:
        return 0

    opcja = f"{'usunEtykiete_' if delete else 'oznaczEtykieta_'}{tag_id}",

    data = {"opcja_zaznaczone_g": opcja}

    for index, message_id in enumerate(messages):
        data[f"wiadomosci[{index}]"] = message_id

    url = "https://synergia.librus.pl/" + (
        "wiadomosci_archiwum" if archive else "wiadomosci_aktualne")

    response = requests.post(
        url, cookies=cookies, data=data, headers={"Referer": url})

    response = bs4.BeautifulSoup(response.text, "html.parser")

    text = response.select_one("div.container.green > div.container-background > p").text

    return int(text.split(": ")[1])


def get_tags(cookies: dict, name_as_key: bool = False) -> dict:
    url = "https://synergia.librus.pl/wiadomosci_aktualne"

    response = bs4.BeautifulSoup(requests.get(url, cookies=cookies).text, "html.parser")

    tag_table_row = response.select("table.message-labels tr:has(td.label-name)")

    tags = {}

    for tag in tag_table_row:
        a_name_element = tag.select_one("td.label-name > a")
        tag_id = a_name_element.attrs["href"][21:]
        tag_name = a_name_element.text.strip()
        tag_color = tag.select_one("td.label-color > span").attrs["style"][19:]

        if not name_as_key:
            tags[tag_id] = {"color": tag_color, "name": tag_name}
            continue

        tags[tag_name] = {"color": tag_color, "id": tag_id}

    return tags


def create_tag(cookies: dict, tag_name: str, tag_color: str) -> bool:
    url = "https://synergia.librus.pl/wiadomosci/etykiety"

    data = {
        "nazwaEtykiety": tag_name, "rgbKoloru": tag_color, "dodaj_etykiete": "Dodaj"}

    response = requests.post(url, data=data, cookies=cookies, headers={"Referer": url})

    if "Etykieta o podanej nazwie już istnieje." in response.text:
        return False

    elif "Etykieta została dodana" in response.text:
        return True

    raise Exception("tag was not created due to unknown reasons")


def delete_tag(cookies: dict, tag_id: str) -> bool:

    url = "https://synergia.librus.pl/wiadomosci/etykiety"

    data = {
        "tak": "Tak",
        "potwierdzenie": "Tak",
        "parametr": tag_id,
        "id": "1"
    }

    response = requests.post(
        url, cookies=cookies, headers={"Referer": url}, data=data
    )

    status = bs4.BeautifulSoup(
        response.text, "html.parser").select_one("div.container.green > div > p")

    if status is not None and status.text == "Etykieta została usunięta.":
        return True

    return False



from typing import Iterator
import bs4
import requests
import re


def get_timetable(cookies: dict, *, month: str = "", year: str = "", csrf_token: str = None) -> dict:
    response = requests.post(
        url="https://synergia.librus.pl/terminarz",
        data={
            "requestkey": csrf_token,
            "miesiac": month,
            "rok": year
        },
        cookies=cookies
    )

    response = bs4.BeautifulSoup(
        re.sub("<br ?/>", " ", response.text), "html.parser"
    )

    selected = list(map(
        lambda x: x.attrs["value"],
        response.select("option[selected]"))
    )

    days_data = {
        int(day.select_one("div[class]").text): [
            x.text.strip() for x in day.select("td")
        ] for day in response.select("div.kalendarz-dzien")
    }

    return {
        "date": selected,
        "days": days_data
    }


def get_attendence(cookies: dict) -> Iterator:
    response = bs4.BeautifulSoup(
        requests.get(
            url="https://synergia.librus.pl/przegladaj_nb/uczen",
            cookies=cookies
        ).text,
        "html.parser"
    )

    rows = response.select(
        "tr[class^='line']:not(:first-of-type):not(:last-child)"
    )

    for row in rows:
        entries = [
            {
                (val := x.split(": "))[0]: val[1] for x in
                re.split(" *<br/?> *", re.sub("</?b>", "", entry.attrs["title"]))
            } for entry in row.select("td.center a.ocena")
        ]

        yield {
            "date": row.select_one("td").text.split(" ")[0],
            "entries": entries
        }



import bs4
import requests
import re


def get_notifications(cookies: dict) -> dict:

    response = requests.get(
        "https://synergia.librus.pl/uczen/index",
        cookies=cookies
    )

    response = bs4.BeautifulSoup(
        response.text,
        "html.parser"
    )

    titles = response.select(
        "div[id='graphic-menu'] li:not(:first-child) a[id^='icon'][title]"
    )

    buttons = response.select("a.button.counter")

    keys = [
        title.text.strip(" \n") for title in titles
    ]

    values = [
        x.text.strip(" \n") for x in buttons
    ]

    return dict(zip(keys, values))


def get_user_information(cookies: dict) -> dict:
    response = bs4.BeautifulSoup(
        requests.get(
            "https://synergia.librus.pl/informacja",
            cookies=cookies
        ).text,
        "html.parser"
    )

    rows = response.select(
        "table.decorated.big.center.form > tbody > tr[class^='line']:not(:last-of-type)"
    )

    return {
        row.select_one("th.big").text.strip():
            re.sub(
                r" ?\n +", ", ",
                row.select_one("td").text.strip()
        ) for row in rows[:-1]
    }



import bs4
import requests
from dataclasses import dataclass
import re


@dataclass()
class Grade:
    grade: float
    subject: str
    category: str
    date: str
    teacher: str
    mean: bool
    to_max_points: bool
    additional: dict

    @property
    def points(self) -> list[int]:
        maximal = re.findall(r"\(\d+-(\d+)\)", self.category)

        return [self.grade, int(maximal[-1])]


@dataclass()
class Semester():
    subject: str
    grades: list[Grade]

    @property
    def points_sum(self) -> int:
        s = 0
        for grade in self.grades:
            if grade.mean:
                s += grade.grade

        return s

    @property
    def points_max(self) -> int:
        s = 0
        for grade in self.grades:
            if grade.mean and grade.to_max_points:
                s += grade.points[1]

        return s

    @property
    def percentage(self) -> float:
        return self.points_sum / self.points_max * 100


@dataclass()
class Subject:
    name: str
    semesters: list[Semester]

    @property
    def grades(self) -> list[Grade]:
        grades = []
        for semester in self.semesters:
            grades += semester.grades

        return grades

    @property
    def points_sum(self) -> int:
        return sum(x.points_sum for x in self.semesters)

    @property
    def points_max(self) -> int:
        return sum(x.points_max for x in self.semesters)

    @property
    def percentage(self) -> float:
        return self.points_sum / self.points_max * 100


def get_grades_detailed(cookies: dict) -> list[Subject]:

    response = bs4.BeautifulSoup(
        requests.get(
            "https://synergia.librus.pl/przegladaj_oceny/uczen",
            cookies=cookies
        ).text,
        "html.parser"
    )

    lines = response.select("table.decorated.stretch > tbody > tr[class^='line']:not(tr[id])")

    subjects = []

    for line in lines:
        cols = line.select("td:has(span.grade-box):not(td.center)")

        if not cols:
            continue

        semesters = []

        subject_name = line.select_one("td:not(td[class])").text

        for semester in cols:
            a_elements = semester.select("span > a")
            grades_obj = []

            for a in a_elements:
                grade_info = dict(re.findall(
                    "([^:>]+): +([^<]*)",
                    a.attrs["title"]
                ))

                content = ['Kategoria', 'Data', 'Nauczyciel']

                is_eq_to = [
                    ('Licz do wyniku', "TAK"),
                    ('Licz do puli', "TAK"),
                ]

                values = [
                    grade_info.get(x, "") for x in content
                ]

                values += [
                    grade_info.get(x[0], "NIE") == x[1] for x in is_eq_to
                ]

                grades_obj.append(
                    Grade(
                        float(a.text), subject_name, *values,
                        additional={
                            key: grade_info[key] for key in grade_info if key not in content + [x[0] for x in is_eq_to]
                        }
                    )
                )

            semesters.append(
                Semester(subject_name, grades_obj)
            )

        subjects.append(
            Subject(
                subject_name,
                semesters
            )
        )

    return subjects


def get_grades_head(thead: str) -> dict:

    thead = bs4.BeautifulSoup(
        thead,
        "html.parser"
    )

    rows = thead.select("tr")

    rows_dict = {i: [] for i in range(len(rows))}

    for i, row in enumerate(rows):

        cells = row.select("th, td")

        for cell in cells:

            colspan = int(cell.attrs.get("colspan", 1))
            rowspan = int(cell.attrs.get("rowspan", 1))

            for row_index in range(i, i + rowspan):

                cell_text = cell.attrs.get(
                    "title", cell.text).strip()

                rows_dict[row_index] += [
                    cell_text.replace("<br>", "") for _ in range(colspan)
                ]

    return rows_dict


def get_grades(cookies: dict) -> list:

    response = bs4.BeautifulSoup(
        requests.get(
            "https://synergia.librus.pl/przegladaj_oceny/uczen",
            cookies=cookies
        ).text,
        "html.parser"
    )

    thead = list(get_grades_head(
        str(response.select_one("table.decorated.stretch > thead"))
    ).values())[-1]

    lines = response.select(
        "table.decorated.stretch > tbody > tr[class^='line']:not(tr[id])"
    )

    subjects = []

    for line in lines:

        cells = [
            c.text.replace("\n", " ").strip()
            for c in line.select("td")
        ]

        cells = list(zip(thead, cells))

        subjects.append(
            list(filter(all, cells))
        )

    return subjects



import requests
import bs4


def get_announcements(cookies: dict) -> list:
    response = bs4.BeautifulSoup(
        requests.get(url="https://synergia.librus.pl/ogloszenia", cookies=cookies).text,
        "html.parser"
    )

    head = ["temat", "dodal", "data", "tresc"]

    announcements = [dict(zip(head, map(
        lambda x: x.text, annoucement.select("td:not(tfoot > tr > td)")
    ))) for annoucement in response.select("table.decorated.big.center.printable.margin-top")]

    return announcements



from setuptools import setup, find_packages

VERSION = '0.0.1'
DESCRIPTION = 'Web Scraper do serwisu Librus Synergia'

setup(
    name="librus_scraper",
    version=VERSION,
    description=DESCRIPTION,
    author="Piotr Ginał",
    packages=["librus_scraper"],
    install_requires=[
        "beautifulsoup4",
        "requests",
    ],
    keywords='scraper librus librus-synergia'
)
